/**
 * Copyright (c) 2010 Alois Glomann gnalff@lucubrators.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package net.lucubrators.honeycomb.guice.controller;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import net.lucubrators.honeycomb.xml.controller.DefaultControllerProvider;
import net.lucubrators.honeycomb.xml.controller.exceptions.UnknownControllerException;

import com.google.inject.Injector;
import com.google.inject.Key;

public class GuicedControllerProvider extends DefaultControllerProvider {

	private static Logger	log	= LoggerFactory.getLogger(GuicedControllerProvider.class);

	private Injector		injector;

	public void init(Injector injector) {
		this.injector = injector;
	}

	@Override
	public Object getController(String id) throws UnknownControllerException {
		Object controller = super.getController(id);
		if (Key.class.isAssignableFrom(controller.getClass())) {
			log.trace("Controller {} is of type key. Getting instance via injector", id);
			return injector.getInstance((Key<?>) controller);
		}
		log.trace("Controller {} is not of type key. Returning the instance directly", id);
		return controller;
	}
}
